import json
import requests

from .helpers import check_credentials, check_required_fields, hash_pasword


class Faspay(object):
    """
    Docs https://faspay.co.id/docs/index-business.html#introduction

    faspay = Faspay()
    faspay.user_id = 'xxxxx'
    faspay.mode = 'Development' or 'Production'
    faspay.merchant_id = 'botxxxxx'
    faspay.merchant_password = 'xxxxxxxx'
    faspay.merchant_name = 'examplestore.com'
    """
    mode = 'Development'
    user_id = None
    merchant_id = None
    merchant_password = None
    merchant_name = None

    payment_channel_inquiry_url = '/cvr/100001/10'
    post_data_url = '/cvr/300011/10'
    payment_status_url = '/cvr/100004/10'
    cancel_transaction_url = '/cvr/100005/10'

    @property
    def api_url(self):
        if self.mode == 'Development' or self.mode == 'development':
            return 'https://dev.faspay.co.id'
        elif self.mode == 'Production' or self.mode == 'production':
            return 'https://web.faspay.co.id'
        else:
            raise ValueError("You must set `mode` to either `Development` or `Production`")

    def send(self, endpoint, to_json=True, **kwargs):
        response_data = dict()

        headers = {'Accept': 'application/json',
                   'Content-Type': 'application/json'}
        url = self.api_url + endpoint
        response = requests.post(url, headers=headers, data=json.dumps(kwargs))
        response_data.update({'status_code': response.status_code})

        if to_json:
            response_data.update(**response.json())
        else:
            response_data.update({'result': response.text})

        return response_data

    def payment_channel_inquiry(self, **kwargs):
        check_credentials(self.user_id, self.merchant_id, self.merchant_name, self.merchant_password)

        endpoint = self.payment_channel_inquiry_url

        request_data = dict()
        request_data['request'] = kwargs.get('request', 'Daftar Payment Channel') 
        request_data['merchant_id'] = self.merchant_id
        request_data['merchant'] = self.merchant_name

        signature = [
            self.user_id,
            self.merchant_password
        ]

        request_data.update({'signature': hash_pasword(signature)})

        return self.send(endpoint, **request_data)

    def post_data(self, **kwargs):
        check_credentials(self.user_id, self.merchant_id, self.merchant_name, self.merchant_password)

        endpoint = self.post_data_url

        request_data = dict()
        request_data.update(**kwargs)
        request_data['request'] = kwargs.get('request', 'Transmisi Info Detil Pembelian')
        request_data['merchant_id'] = self.merchant_id
        request_data['merchant'] = self.merchant_name

        required_fields = ['bill_no', 'bill_date', 'bill_expired', 'bill_desc', 'bill_currency', 'bill_total', 
                           'payment_channel', 'pay_type', 'cust_no', 'cust_name', 'msisdn', 'email', 'terminal', 'item']
        existing_fields = request_data.keys()
        check_required_fields(required_fields, existing_fields)

        signature = [
            self.user_id,
            self.merchant_password,
            request_data['bill_no']
        ]

        request_data.update({'signature': hash_pasword(signature)})

        return self.send(endpoint, **request_data)

    def inquiry_payment_status(self, **kwargs):
        check_credentials(self.user_id, self.merchant_id, self.merchant_name, self.merchant_password)

        endpoint = self.payment_status_url

        request_data = dict()
        request_data.update(**kwargs)
        request_data['merchant_id'] = self.merchant_id

        required_fields = ['trx_id', 'bill_no']
        existing_fields = request_data.keys()
        check_required_fields(required_fields, existing_fields)

        signature = [
            self.user_id,
            self.merchant_password,
            request_data['bill_no']
        ]

        request_data.update({'signature': hash_pasword(signature)})

        return self.send(endpoint, **request_data)

    def cancel_transaction(self, **kwargs):
        check_credentials(self.user_id, self.merchant_id, self.merchant_name, self.merchant_password)

        endpoint = self.cancel_transaction_url

        request_data = dict()
        request_data.update(**kwargs)
        request_data['request'] = kwargs.get('request', 'Canceling Payment')
        request_data['merchant_id'] = self.merchant_id
        request_data['merchant'] = self.merchant_name

        required_fields = ['trx_id', 'bill_no', 'payment_cancel']
        existing_fields = request_data.keys()
        check_required_fields(required_fields, existing_fields)

        signature = [
            self.user_id,
            self.merchant_password,
            request_data['bill_no']
        ]

        request_data.update({'signature': hash_pasword(signature)})

        return self.send(endpoint, **request_data)
